@@ -1,25 +1,26 @@
 /*----------------------------------------------------------------------------
-  ChucK Concurrent, On-the-fly Audio Programming Language
-    Compiler and Virtual Machine
+  ChucK Strongly-timed Audio Programming Language
+    Compiler, Virtual Machine, and Synthesis Engine
 
-  Copyright (c) 2004 Ge Wang and Perry R. Cook.  All rights reserved.
+  Copyright (c) 2003 Ge Wang and Perry R. Cook. All rights reserved.
     http://chuck.stanford.edu/
     http://chuck.cs.princeton.edu/
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
+  it under the dual-license terms of EITHER the MIT License OR the GNU
+  General Public License (the latter as published by the Free Software
+  Foundation; either version 2 of the License or, at your option, any
+  later version).
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful and/or
+  interesting, but WITHOUT ANY WARRANTY; without even the implied warranty
+  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+  MIT Licence and/or the GNU General Public License for details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  U.S.A.
+  You should have received a copy of the MIT License and the GNU General
+  Public License (GPL) along with this program; a copy of the GPL can also
+  be obtained by writing to the Free Software Foundation, Inc., 59 Temple
+  Place, Suite 330, Boston, MA 02111-1307 U.S.A.
 -----------------------------------------------------------------------------*/
 
 //-----------------------------------------------------------------------------
@@ -30,10 +31,11 @@
 // date: Autumn 2002
 //-----------------------------------------------------------------------------
 #include "chuck_absyn.h"
-#include "chuck_utils.h"
 #include "chuck_errmsg.h"
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
+#include <string> // 1.5.1.5 for string concat
 
 
 // 1.5.0.5 (ge) option to include in case we need something from flex/bison
@@ -342,23 +344,116 @@ a_Stmt new_stmt_from_case( a_Exp exp, uint32_t lineNum, uint32_t posNum )
     return a;
 }
 
+a_Stmt new_stmt_from_import( a_Import list, uint32_t line, uint32_t where ) // 1.5.4.0 (ge) added
+{
+    a_Stmt a = (a_Stmt)checked_malloc( sizeof(struct a_Stmt_) );
+    a->s_type = ae_stmt_import;
+    a->stmt_import.list = list;
+    a->line = line; a->where = where;
+    a->stmt_import.line = line; a->stmt_import.where = where;
+    a->stmt_import.self = a;
+
+    return a;
+}
+
+a_Stmt new_stmt_from_doc( a_Doc list, uint32_t line, uint32_t where ) // 1.5.4.4 (ge) added
+{
+    a_Stmt a = (a_Stmt)checked_malloc( sizeof(struct a_Stmt_) );
+    a->s_type = ae_stmt_doc;
+    a->stmt_doc.list = list;
+    a->line = line; a->where = where;
+    a->stmt_doc.line = line; a->stmt_doc.where = where;
+    a->stmt_doc.self = a;
+
+    return a;
+}
+
+a_Import new_import( c_str str, a_Id_List list, uint32_t line, uint32_t where ) // 1.5.4.0 (ge) added
+{
+    a_Import a = (a_Import)checked_malloc( sizeof(struct a_Import_) );
+
+    // check which option
+    if( str )
+    {
+        a->what = str; // no strdup( str ); <-- str should have been allocated in alloc_str()
+    }
+    else // id list
+    {
+        a_Id_List curr = list;
+        std::string result = "";
+
+        // iterate over id list
+        while( curr )
+        {
+            result += S_name(curr->xid);
+            // if not the last, appent DOT
+            if( curr->next ) result += ".";
+            // set to next
+            curr = curr->next;
+        }
+
+        // sum of string lengths, +1 for null terminator
+        size_t len = result.length() + 1;
+        // allocate
+        char * sc = (char *)checked_malloc( len );
+        // copy
+        strncpy( sc, result.c_str(), len );
+        // set
+        a->what = sc;
+        // clean up id list, since we will not have references to it after this
+        delete_id_list( list );
+    }
+
+    // set line info
+    a->line = line; a->where = where;
+
+    return a;
+}
+
+a_Import prepend_import( a_Import target, a_Import list, uint32_t lineNum, uint32_t posNum )
+{
+    target->next = list;
+    return target;
+}
+
+a_Doc new_doc( c_str str, uint32_t line, uint32_t where ) // 1.5.4.4 (ge) added
+{
+    a_Doc a = (a_Doc)checked_malloc( sizeof(struct a_Doc_) );
+
+    // copy allocated string pointer
+    a->desc = str; // no strdup( str ); <-- str should have been allocated in alloc_str()
+
+    // set line info
+    a->line = line; a->where = where;
+
+    return a;
+}
+
+a_Doc prepend_doc( a_Doc target, a_Doc list, uint32_t lineNum, uint32_t posNum )
+{
+    target->next = list;
+    return target;
+}
+
 a_Exp append_expression( a_Exp list, a_Exp exp, uint32_t lineNum, uint32_t posNum )
 {
-  a_Exp current;
-  current = list->next;
-  if (current == NULL) {
-    list->next = exp;
-    return list;
-  }
-
-  while (1)
+    a_Exp current;
+    current = list->next;
+    if( current == NULL )
     {
-      if (current->next == NULL) {
-        current->next = exp;
-        break;
-      } else {
-        current = current->next;
-      }
+        list->next = exp;
+        return list;
+    }
+
+    while( true )
+    {
+        if( current->next == NULL ) {
+            current->next = exp;
+            break;
+        }
+        else {
+            current = current->next;
+        }
     }
     return list;
 }
@@ -393,13 +488,16 @@ a_Exp new_exp_from_unary( ae_Operator oper, a_Exp exp, uint32_t lineNum, uint32_
 }
 
 a_Exp new_exp_from_unary2( ae_Operator oper, a_Type_Decl type,
-                           a_Array_Sub array, uint32_t lineNum, uint32_t posNum )
+                           int ctor_invoked, a_Exp ctor_args, a_Array_Sub array,
+                           uint32_t lineNum, uint32_t posNum )
 {
     a_Exp a = (a_Exp)checked_malloc( sizeof( struct a_Exp_ ) );
     a->s_type = ae_exp_unary;
     a->s_meta = ae_meta_value;
     a->unary.op = oper;
     a->unary.type = type;
+    a->unary.ctor.invoked = ctor_invoked;
+    a->unary.ctor.args = ctor_args;
     a->unary.array = array;
     a->line = lineNum; a->where = posNum;
     a->unary.line = lineNum; a->unary.where = posNum;
@@ -520,7 +618,7 @@ a_Exp new_exp_from_id( c_str xid, uint32_t lineNum, uint32_t posNum )
     return a;
 }
 
-a_Exp new_exp_from_int( long num, uint32_t lineNum, uint32_t posNum )
+a_Exp new_exp_from_int( t_CKINT num, uint32_t lineNum, uint32_t posNum )
 {
     a_Exp a = (a_Exp)checked_malloc( sizeof( struct a_Exp_ ) );
     a->s_type = ae_exp_primary;
@@ -534,7 +632,7 @@ a_Exp new_exp_from_int( long num, uint32_t lineNum, uint32_t posNum )
     return a;
 }
 
-a_Exp new_exp_from_float( double num, uint32_t lineNum, uint32_t posNum )
+a_Exp new_exp_from_float( t_CKFLOAT num, uint32_t lineNum, uint32_t posNum )
 {
     a_Exp a = (a_Exp)checked_malloc( sizeof( struct a_Exp_ ) );
     a->s_type = ae_exp_primary;
@@ -583,10 +681,10 @@ a_Exp new_exp_from_array_lit( a_Array_Sub exp_list, uint32_t lineNum, uint32_t p
     a->s_meta = ae_meta_value;
     a->primary.s_type = ae_primary_array;
     a->primary.array = exp_list;
+    a->primary.array->self = a; // 1.5.2.0 (ge & nshaheed) added
     a->line = lineNum; a->where = posNum;
     a->primary.line = lineNum; a->primary.where = posNum;
     a->primary.self = a;
-
     return a;
 }
 
@@ -606,24 +704,24 @@ a_Exp new_exp_from_if( a_Exp cond, a_Exp if_exp, a_Exp else_exp, uint32_t lineNu
     return a;
 }
 
-a_Exp new_exp_decl_external( a_Type_Decl type, a_Var_Decl_List var_decl_list, int is_static, uint32_t lineNum, uint32_t posNum )
+a_Exp new_exp_decl_external( a_Type_Decl type, a_Var_Decl_List var_decl_list, int is_static, int is_const, uint32_t lineNum, uint32_t posNum )
 {
-    a_Exp a = new_exp_decl( type, var_decl_list, is_static, lineNum, posNum );
+    a_Exp a = new_exp_decl( type, var_decl_list, is_static, is_const, lineNum, posNum );
     a->decl.is_global = 1;
     EM_error2( posNum, "'external' keyword is deprecated. please use 'global'" );
 
     return a;
 }
 
-a_Exp new_exp_decl_global( a_Type_Decl type, a_Var_Decl_List var_decl_list, int is_static, uint32_t lineNum, uint32_t posNum )
+a_Exp new_exp_decl_global( a_Type_Decl type, a_Var_Decl_List var_decl_list, int is_static, int is_const, uint32_t lineNum, uint32_t posNum )
 {
-    a_Exp a = new_exp_decl( type, var_decl_list, is_static, lineNum, posNum );
+    a_Exp a = new_exp_decl( type, var_decl_list, is_static, is_const, lineNum, posNum );
     a->decl.is_global = 1;
 
     return a;
 }
 
-a_Exp new_exp_decl( a_Type_Decl type, a_Var_Decl_List var_decl_list, int is_static, uint32_t lineNum, uint32_t posNum )
+a_Exp new_exp_decl( a_Type_Decl type, a_Var_Decl_List var_decl_list, int is_static, int is_const, uint32_t lineNum, uint32_t posNum )
 {
     a_Exp a = (a_Exp)checked_malloc( sizeof( struct a_Exp_ ) );
     a->s_type = ae_exp_decl;
@@ -632,6 +730,7 @@ a_Exp new_exp_decl( a_Type_Decl type, a_Var_Decl_List var_decl_list, int is_stat
     a->decl.var_decl_list = var_decl_list;
     a->line = lineNum; a->where = posNum;
     a->decl.is_static = is_static;
+    a->decl.is_const = is_const; // 1.5.1.3
     a->decl.line = lineNum; a->decl.where = posNum;
     a->decl.self = a;
 
@@ -710,10 +809,12 @@ a_Exp new_exp_from_nil( uint32_t lineNum, uint32_t posNum )
     return a;
 }
 
-a_Var_Decl new_var_decl( c_constr xid, a_Array_Sub array, uint32_t lineNum, uint32_t posNum )
+a_Var_Decl new_var_decl( c_constr xid, int ctor_invoked, a_Exp ctor_args, a_Array_Sub array, uint32_t lineNum, uint32_t posNum )
 {
     a_Var_Decl a = (a_Var_Decl)checked_malloc( sizeof( struct a_Var_Decl_ ) );
     a->xid = insert_symbol(xid);
+    a->ctor.invoked = ctor_invoked;
+    a->ctor.args = ctor_args;
     a->array = array;
     a->line = lineNum; a->where = posNum;
 
@@ -788,7 +889,8 @@ a_Func_Def new_func_def( ae_Keyword func_decl, ae_Keyword static_decl,
         sizeof( struct a_Func_Def_ ) );
     a->func_decl = func_decl;
     a->static_decl = static_decl;
-    a->type_decl = type_decl;
+    // substitute if NULL | 1.5.2.0 (ge) for constructors
+    a->type_decl = type_decl ? type_decl : new_type_decl(new_id_list("void",0,0),0,0,0);
     a->name = insert_symbol( name );
     a->arg_list = arg_list;
     a->s_type = ae_func_user;
@@ -799,6 +901,32 @@ a_Func_Def new_func_def( ae_Keyword func_decl, ae_Keyword static_decl,
     return a;
 }
 
+// @operator overload | 1.5.1.5
+a_Func_Def new_op_overload( ae_Keyword func_decl, ae_Keyword static_decl,
+                            a_Type_Decl type_decl, ae_Operator oper,
+                            a_Arg_List arg_list, a_Stmt code,
+                            uint32_t is_from_ast, uint32_t overload_post,
+                            uint32_t lineNum, uint32_t posNum, uint32_t operPos )
+{
+    a_Func_Def a = (a_Func_Def)checked_malloc(
+        sizeof( struct a_Func_Def_ ) );
+    a->func_decl = func_decl;
+    a->static_decl = static_decl;
+    a->type_decl = type_decl;
+    // construct op overload function name
+    std::string s( "@operator" ); s += op2str(oper);
+    a->name = insert_symbol( s.c_str() );
+    a->op2overload = oper;
+    a->arg_list = arg_list;
+    a->s_type = ae_func_user;
+    a->code = code;
+    a->ast_owned = is_from_ast != 0; // 1.5.0.5 (ge) added
+    a->overload_post = overload_post; // 1.5.1.5 (ge) added
+    a->line = lineNum; a->where = posNum; a->operWhere = operPos;
+
+    return a;
+}
+
 a_Class_Def new_class_def( ae_Keyword class_decl, a_Id_List name,
                            a_Class_Ext ext, a_Class_Body body, uint32_t lineNum, uint32_t posNum )
 {
@@ -977,7 +1105,11 @@ a_Vec new_vec( a_Exp e, uint32_t lineNum, uint32_t posNum ) // ge: added 1.3.5.3
 }
 
 
+
+
+// operator strings
 static const char * op_str[] = {
+  "[no_op]",
   "+",
   "-",
   "*",
@@ -1013,23 +1145,31 @@ static const char * op_str[] = {
   "!",
   "@=>",
   "=<",
-  "^=>",
+  "=^",
+  "=v",
+  "=",
+  "$",
+  "@@",
+  "::",
   "spork ~",
   "typeof",
   "sizeof",
   "new",
   "<-",
-  "->"
+  "->",
+  "<--",
+  "-->",
+  ">--",
+  "--<"
 };
-
 //-----------------------------------------------------------------------------
 // name: op2str()
-// desc: chuck operator to string
+// desc: operator enumeration to string
 //-----------------------------------------------------------------------------
 const char * op2str( ae_Operator op )
 {
     t_CKINT index = (t_CKINT)op;
-    if( index < 0 || index >= (t_CKINT)(sizeof( op_str )/sizeof( char * )) )
+    if( index < 0 || index >= ae_op_count )
         return "[non-existent operator]";
 
     return op_str[index];
@@ -1038,6 +1178,48 @@ const char * op2str( ae_Operator op )
 
 
 
+//-----------------------------------------------------------------------------
+// name: str2op()
+// desc: string to operator enumeration
+//-----------------------------------------------------------------------------
+ae_Operator str2op( const char * str )
+{
+    std::string s(str);
+    for( t_CKUINT i = 1; i < ae_op_count; i++ )
+        if( s == op_str[i] ) return (ae_Operator)i;
+
+    return ae_op_none;
+}
+
+
+
+
+// function pointer kind names | 1.5.2.0
+static const char * fpkind_str[] = {
+    "[unknown]",
+    "ctor",
+    "dtor",
+    "mfun",
+    "sfun",
+    "gfun",
+    "addr"
+};
+//-----------------------------------------------------------------------------
+// name: fpkind2str() | 1.5.2.0
+// desc: convert fp kind by enum to str
+//-----------------------------------------------------------------------------
+const char * fpkind2str( ae_FuncPointerKind kind )
+{
+    t_CKINT index = (t_CKINT)kind;
+    if( index < 0 || index >= ae_fp_count )
+        return "[non-existent func kind]";
+
+    return fpkind_str[index];
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // AST cleanup function (these are mostly called by chuck compiler)
 //-----------------------------------------------------------------------------
@@ -1219,6 +1401,12 @@ void delete_stmt( a_Stmt stmt )
     case ae_stmt_gotolabel:
         delete_stmt_from_label( stmt );
         break;
+    case ae_stmt_import:
+        delete_stmt_from_import( stmt );
+        break;
+    case ae_stmt_doc: // 1.5.4.4 (ge) added
+        delete_stmt_from_doc( stmt );
+        break;
     }
 
     CK_SAFE_FREE( stmt );
@@ -1301,6 +1489,48 @@ void delete_stmt_from_label( a_Stmt stmt )
     // TODO: someting with S_Symbol stmt->gotolabel.name
 }
 
+void delete_stmt_from_import( a_Stmt stmt )
+{
+    EM_log( CK_LOG_FINEST, "deleting stmt %p (import)...", (void *)stmt );
+
+    // pointer
+    a_Import next = NULL, i = stmt->stmt_import.list;
+
+    // iterate instead of recurse to avoid stack overflow
+    while( i )
+    {
+        // delete the content
+        CK_SAFE_FREE( i->what );
+        // get next before we delete this one
+        next = i->next;
+        // delete the import target
+        CK_SAFE_FREE( i );
+        // move to the next one
+        i = next;
+    }
+}
+
+void delete_stmt_from_doc( a_Stmt stmt )
+{
+    EM_log( CK_LOG_FINEST, "deleting stmt %p (doc)...", (void *)stmt );
+
+    // pointer
+    a_Doc next = NULL, i = stmt->stmt_doc.list;
+
+    // iterate instead of recurse to avoid stack overflow
+    while( i )
+    {
+        // delete the content
+        CK_SAFE_FREE( i->desc );
+        // get next before we delete this one
+        next = i->next;
+        // delete the import target
+        CK_SAFE_FREE( i );
+        // move to the next one
+        i = next;
+    }
+}
+
 void delete_exp_from_primary( a_Exp_Primary_ & p )
 {
     EM_log( CK_LOG_FINEST, "deleting exp (primary)..." );
@@ -1406,6 +1636,9 @@ void delete_exp( a_Exp e )
     {
         // TODO: release reference type
         // TODO: release reference owner
+        // 1.5.4.4 (ge) actually -- the above may not be necessary if:
+        // 1) they are not ref-counted in the first place; e.g., see type_engine_check_exp() which assigned `type`
+        // AND 2) the AST is cleaned up before types and nspcs etc.
 
         // delete content in this exp
         delete_exp_contents( e );
@@ -1507,7 +1740,13 @@ void delete_exp_decl( a_Exp e )
 void delete_exp_from_id( a_Exp_Primary e )
 {
     // TODO: do we need to anything with the Symbol?
+    // 1.5.4.4 (ge) symbols are additive but also also unique per string thus growth bounded -- can keep around, in practice
     EM_log( CK_LOG_FINEST, "deleting exp (primary ID '%s') [%p]...", S_name(e->var), (void *)e );
+
+    // TODO: release reference func_alias #2024-ctor-this
+    // 1.5.4.4 (ge) actually -- the above may not be necessary if:
+    // 1) func_alisa not ref-counted in the first place; e.g., see xxx() which assigned `func_alias`
+    // AND 2) the AST is cleaned up before the func etc.
 }
 
 void delete_exp_from_str( a_Exp_Primary e )
