@@ -1,25 +1,26 @@
 /*----------------------------------------------------------------------------
-  ChucK Concurrent, On-the-fly Audio Programming Language
-    Compiler and Virtual Machine
+  ChucK Strongly-timed Audio Programming Language
+    Compiler, Virtual Machine, and Synthesis Engine
 
-  Copyright (c) 2004 Ge Wang and Perry R. Cook.  All rights reserved.
+  Copyright (c) 2003 Ge Wang and Perry R. Cook. All rights reserved.
     http://chuck.stanford.edu/
     http://chuck.cs.princeton.edu/
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
+  it under the dual-license terms of EITHER the MIT License OR the GNU
+  General Public License (the latter as published by the Free Software
+  Foundation; either version 2 of the License or, at your option, any
+  later version).
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful and/or
+  interesting, but WITHOUT ANY WARRANTY; without even the implied warranty
+  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+  MIT Licence and/or the GNU General Public License for details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  U.S.A.
+  You should have received a copy of the MIT License and the GNU General
+  Public License (GPL) along with this program; a copy of the GPL can also
+  be obtained by writing to the Free Software Foundation, Inc., 59 Temple
+  Place, Suite 330, Boston, MA 02111-1307 U.S.A.
 -----------------------------------------------------------------------------*/
 
 //-----------------------------------------------------------------------------
@@ -243,7 +244,8 @@ public:
     t_CKBOOL stopListeningForGlobalEvent( const char * name, void (*callback)(const char*) );
     t_CKBOOL stopListeningForGlobalEvent( const char * name, t_CKINT callbackID, void (*callback)(t_CKINT) );
 
-    t_CKBOOL getGlobalUGenSamples( const char * name, SAMPLE* buffer, int numFrames );
+    t_CKBOOL getGlobalUGenSamples( const char * name, SAMPLE * buffer, int numFrames );
+    t_CKBOOL getGlobalUGenSamplesMulti( const char * name, SAMPLE * buffer, int numFrames, int numChannels );
 
     t_CKBOOL setGlobalIntArray( const char * name, t_CKINT arrayValues[], t_CKUINT numValues );
     t_CKBOOL getGlobalIntArray( const char * name, void (*callback)(t_CKINT[], t_CKUINT) );
@@ -305,13 +307,20 @@ public:
     Chuck_UGen* get_global_ugen( const std::string & name );
     Chuck_UGen** get_ptr_to_global_ugen( const std::string & name );
 
+    // global array (internal; audio thread)
     t_CKBOOL init_global_array( const std::string & name, Chuck_Type* type, te_GlobalType arr_type );
+    // global array get (internal; audio thread)
     Chuck_Object* get_global_array( const std::string & name );
     Chuck_Object** get_ptr_to_global_array( const std::string & name );
     t_CKINT get_global_int_array_value( const std::string & name, t_CKUINT index );
     t_CKINT get_global_associative_int_array_value( const std::string & name, const std::string & key );
     t_CKFLOAT get_global_float_array_value( const std::string & name, t_CKUINT index );
     t_CKFLOAT get_global_associative_float_array_value( const std::string & name, const std::string & key );
+    // global array set (internal; audio thread)
+    t_CKBOOL set_global_int_array( const std::string & name, t_CKINT arrayValues[], t_CKUINT numValues );
+    t_CKBOOL set_global_int_array_value( const std::string & name, t_CKUINT index, t_CKINT value );
+    t_CKBOOL set_global_float_array( const std::string & name, t_CKFLOAT arrayValues[], t_CKUINT numValues );
+    t_CKBOOL set_global_float_array_value( const std::string & name, t_CKUINT index, t_CKFLOAT value );
 
     t_CKBOOL init_global_object( const std::string & name, Chuck_Type* type );
     t_CKBOOL is_global_object_init( const std::string & name );
@@ -349,7 +358,7 @@ private:
     std::map< std::string, Chuck_Global_Object_Container* > m_global_objects;
 
     // global variables -- messages for set, get, etc.
-    XCircleBuffer< Chuck_Global_Request > m_global_request_queue;
+    FinalRingBuffer< Chuck_Global_Request > m_global_request_queue;
 
     // retry queue: sometimes a request comes through exactly 1 sample
     // before the relevant global variable is constructed
@@ -357,7 +366,7 @@ private:
     // in these cases, retrying 1 sample later usually works.
     // this is ok because the external host has no guarantee of sample-level
     // determinism, like we have within the ChucK VM
-    XCircleBuffer< Chuck_Global_Request > m_global_request_retry_queue;
+    FinalRingBuffer< Chuck_Global_Request > m_global_request_retry_queue;
 };
 
 
